@@ -19,12 +19,15 @@ class RWKVPTOps(RWKVOp.module):
             a = inquirer.prompt(q)
             dtype = a['type']
         self.dtype = dtype
+        self.runtimedtype = kwargs.get('runtimedtype', dtype)
         # self.sample = torchsample
 
         def initTensor(x):
-            result = x.to(dtype=self.dtype)
+            if len(x.squeeze().shape) == 2:
+                return x.to(self.dtype).t()
+            else:
+                return x.to(self.runtimedtype)
 
-            return result
 
         self.initTensor = initTensor
         self.intTensor = lambda x: torch.tensor(
@@ -36,7 +39,6 @@ class RWKVPTOps(RWKVOp.module):
         self.minimum = torch.minimum
         self.unsqueeze = torch.unsqueeze
         self.expand = lambda x, y: x.expand(*y)
-        self.range = torch.arange
         self.pow = torch.pow
         self.sqrt = torch.sqrt
         self.mean = torch.mean
@@ -50,7 +52,7 @@ class RWKVPTOps(RWKVOp.module):
         def lenn(x): return x.shape[0]
         self.len = lenn
         self.cat = torch.cat
-        def matvec(x, y): return torch.matmul(x, y.t()).t()
+        def matvec(x, y): return torch.matmul(y.to(x.dtype), x).to(y.dtype)
         self.matvec = matvec
 
         def prod(x): return torch.prod(x, dim=1)
@@ -61,20 +63,27 @@ class RWKVPTOps(RWKVOp.module):
 
         self.exp = torch.exp
         self.lerp = torch.lerp
-        def rng(x: int): return torch.range(0, x-1).to(dtype=torch.int64)
-        self.rng = rng
+        self.rng = torch.arange
 
-        def emptyarray(x: int): return [torch.ones(0)]*x
+        def emptyarray(x: int): return []
         self.emptyarray = emptyarray
 
         def arrayPush(x: List[torch.Tensor], y, i: int):
-            x[i] = y
-            return x
+            return x + [y]
         self.arrayPush = arrayPush
 
         def arrayGet(x: List[torch.Tensor], i: int): return x[i]
         self.arrayGet = arrayGet
 
+        def scatter(x, y, z):
+            # like tensor[y] = z
+            x[y] = z
+            return x
+        self.scatter = scatter
+
+        self.scatterindices = [torch.tensor(
+            [2, 3, 4]), torch.tensor([2, 3]), torch.tensor([0, 1])]
+
         def pop(x): return x[-1]
         self.pop = pop
 
@@ -99,12 +108,12 @@ class RWKVPTOps(RWKVOp.module):
             o = w*(xee2/x2) + b
 
             return o
-            # nan on cuda
+
             # return torch.nn.functional.layer_norm(x, x.shape, w.expand(x.shape), b.expand(x.shape))
 
         self.layernorm = ln
         self.emptyState = torch.tensor(self.emptyState, dtype=self.dtype)
-
+        self.stackEmb = True
         self.TensorType = torch.Tensor
         self.MatrixType = torch.Tensor
         self.VectorType = torch.Tensor
@@ -114,8 +123,8 @@ class RWKVPTOps(RWKVOp.module):
 
 
 class RWKVCudaOps(RWKVPTOps):
-    def __init__(self, layers, embed, *args, useGPU=None, runtimedtype=None, **kwargs):
-        super().__init__(layers, embed, *args, **kwargs)
+    def __init__(self, layers, embed, *args, useGPU=None, runtimedtype=None, dev="cuda", **kwargs):
+        super().__init__(layers, embed, *args,runtimedtype=runtimedtype, **kwargs)
         import inquirer
         import torch
 
@@ -135,23 +144,35 @@ class RWKVCudaOps(RWKVPTOps):
         self.runtimedtype = runtimedtype
 
         def initTensor(x):
-            ret = x.to(dtype=self.dtype if len(x.shape) > 1 else runtimedtype).cuda(
-                non_blocking=True)
+            ret = x.to(dtype=self.dtype if len(x.shape) > 1 else runtimedtype).to(dev,
+                                                                                  non_blocking=True)
+
+            if len(ret.squeeze().shape) == 2:
+                ret = ret.t()
             return ret
 
         self.initTensor = initTensor
 
         self.initCpuTensor = lambda x: x.to(dtype=self.runtimedtype)
 
-        def processEmbed(x): return x.to(device='cuda')
+        def processEmbed(x): return x.to(device=dev)
 
         self.processEmbed = processEmbed
 
-        def matvec(x, y): return x.matmul(
-            y.to(dtype=x.dtype).t()).to(dtype=y.dtype).t()
+        def matvec(x, y): return y.to(dtype=x.dtype).matmul(x
+                                                            ).to(dtype=y.dtype)
         self.matvec = matvec
 
-        self.emptyState = self.emptyState.to(dtype=runtimedtype, device='cuda')
+        self.emptyState = self.emptyState.to(dtype=runtimedtype, device=dev)
+
+
+class RWKVMpsOps(RWKVCudaOps):
+
+    def __init__(self, layers, embed, *args, **kwargs):
+        super().__init__(layers, embed, dev="mps", *args, **kwargs)
+        import torch
+        self.lerp = lambda x, y, z: x + (y-x)*z
+        self.roll = lambda x: x[(torch.arange(x.shape[0])-1).relu()]
 
 
 class RWKVPTTSExportOps(RWKVCudaOps):
@@ -202,7 +223,7 @@ class RWKVCudaDeepspeedOps(RWKVCudaOps):
 class RWKVCudaQuantOps(RWKVPTOps):
     import torch
 
-    def __init__(self, layers, embed, *args, runtimedtype=None, dtype=torch.bfloat16, useGPU=None, chunksize=32, preQuantized=False, maxQuantTarget=None, target=None, **kwargs):
+    def __init__(self, layers, embed, *args, runtimedtype=None, dtype=torch.float16, useGPU=None, chunksize=32, preQuantized=False, maxQuantTarget=None, target=None, dev="cuda", **kwargs):
         import torch
         import inquirer
         super().__init__(layers, embed, *args, dtype=dtype, **kwargs)
@@ -211,7 +232,7 @@ class RWKVCudaQuantOps(RWKVPTOps):
             rang = 255
             ran, mini = (x.max(0)[0]-x.min(0)[0])/rang,  x.min(0)[0]
             x = x.double()
-            x = ((x-mini)/ran)
+            x = ((x-mini)/ran).round()
             if stream:
                 x = x.to(
                     dtype=torch.uint8, non_blocking=True).pin_memory()
@@ -229,11 +250,12 @@ class RWKVCudaQuantOps(RWKVPTOps):
 
             yy = yy.to(dtype=dtype)
 
-            xmain = rx.to(dtype=dtype, device=y.device, non_blocking=True).matmul(
-                yy.t()).to(dtype=runtimedtype).t()
+            xmain = yy.matmul(
+                rx.to(dtype=dtype, device=y.device, non_blocking=True)).to(dtype=runtimedtype)
             zp = (y@zpoint).reshape(-1, 1)
             return xmain + zp
-        dev = 'cuda' if (inquirer.confirm(
+        cuda = dev
+        dev = cuda if (inquirer.confirm(
             "Use GPU?", default=True) if useGPU is None else useGPU) else 'cpu'
 
         # target is gb before cpu offload
@@ -256,7 +278,7 @@ class RWKVCudaQuantOps(RWKVPTOps):
         def initTensor(x):
 
             dostream = (torch.cuda.max_memory_reserved(
-                0)/1024/1024/1024 > target) if dev == "cuda" else False
+                0)/1024/1024/1024 > target) if dev == cuda else False
 
             if preQuantized and len(x) == 3:
                 return x[0].to(device=dev), x[1].to(dtype=runtimedtype, device=dev), x[2].to(dtype=runtimedtype, device=dev)
@@ -268,7 +290,7 @@ class RWKVCudaQuantOps(RWKVPTOps):
                 return x.to(dtype=dtype, device=dev)
 
             dontQuantize = (torch.cuda.max_memory_reserved(
-                0)/1024/1024/1024 > maxQuantTarget) if dev == "cuda" else False
+                0)/1024/1024/1024 > maxQuantTarget) if dev == cuda else False
 
             if dontQuantize:
                 if dostream:
@@ -280,7 +302,7 @@ class RWKVCudaQuantOps(RWKVPTOps):
 
             xx = [QuantizeMatrix(x, runtimedtype, dev, dostream)
                   for x in splitmatrices]
-            xxo = torch.cat([x[0] for x in xx], 1)
+            xxo = torch.cat([x[0] for x in xx], 1).t()
             xx1 = torch.cat([x[1] for x in xx])
             xx2 = torch.cat([x[2] for x in xx])
             return xxo, xx1, xx2
@@ -312,6 +334,25 @@ class RWKVStreamBigOps(RWKVCudaQuantOps):
                          chunksize=1, maxQuantTarget=-1, **kwargs)
 
 
+class RWKVQuantMPSOps(RWKVCudaQuantOps):
+
+    def __init__(self, layers, embed, *args, **kwargs):
+        import torch
+        super().__init__(layers, embed, *args, dev="mps", **kwargs)
+
+        self.lerp = lambda x, y, z: x*(1-z)+y*z
+        # roll not implemented in mps, makiing compatible
+        self.roll = lambda x: x[(torch.arange(x.shape[0])-1).relu()]
+
+
+class RWKVStreamMPSOps(RWKVQuantMPSOps):
+
+    def __init__(self, layers, embed, *args, **kwargs):
+
+        super().__init__(layers, embed,  *args, useGPU=True,
+                         chunksize=1, maxQuantTarget=-1, **kwargs)
+
+
 class RWKVSplitCudaOps(RWKVPTOps):
     import torch
 
@@ -322,9 +363,10 @@ class RWKVSplitCudaOps(RWKVPTOps):
 
         devices = inquirer.checkbox(
             'Which devices would you like to use?', choices=['cpu', *[f"cuda:{x}" for x in range(torch.cuda.device_count())]]) if devices is None else devices
+        devices = sorted(devices)[::-1]
 
         self.initTensor = lambda x: x.to(dtype=runtimedtype).cuda() if len(
-            x.shape) == 1 else list(map(lambda zx: zx[1].to(device=devices[zx[0]], dtype=torch.float32 if "cpu" in devices[zx[0]] else torch.bfloat16), enumerate(list(x.chunk(len(devices), dim=1)))))
+            x.shape) == 1 else list(map(lambda zx: zx[1].to(device=devices[zx[0]], dtype=torch.float32 if "cpu" in devices[zx[0]] else torch.bfloat16), enumerate(list(x.t().chunk(len(devices), dim=0)))))
         self.initCpuTensor = lambda x: x.to(dtype=runtimedtype)
 
         # for everything in self, if its a tensor, send to cuda
@@ -335,17 +377,15 @@ class RWKVSplitCudaOps(RWKVPTOps):
 
         def matvec(matx, y):
             chunks = list(map(lambda xx: xx[1].to(
-                device=devices[xx[0]], dtype=matx[xx[0]].dtype, non_blocking=True), enumerate(y.chunk(len(devices), dim=0))))
-            res = matx[0].mv(chunks[0]).to(
-                dtype=runtimedtype, device=y.device, non_blocking=True)
-            for i in range(1, len(chunks)):
-                res = res + matx[i].mv(chunks[i]).to(
+                device=devices[xx[0]], dtype=matx[xx[0]].dtype, non_blocking=True), enumerate(y.chunk(len(devices), dim=1))))
+
+            res = 0
+
+            for i in range(0, len(chunks)):
+                res = res + chunks[i].matmul(matx[i]).to(
                     dtype=runtimedtype, device=y.device, non_blocking=True)
 
             return res
-
-        self.stack = lambda x: x
+        self.processEmbed = lambda x: x.to(device='cuda')
 
         self.matvec = matvec
-        self.layernorm = lambda x, w, b: torch.layer_norm(
-            x.to(device=w.device), w.shape, w, b)
