@@ -1,9 +1,8 @@
 
 from rwkvstic.rwkvMaster import RWKVMaster
-from rwkvstic.agnostic.agnosticRwkv import AgnostigRWKV
+from rwkvstic.agnostic.agnosticRwkv import AgnosticRWKV
 
 from rwkvstic.agnostic.backends.torch import RWKVCudaQuantOps
-from rwkvstic.agnostic.agnosticRwkvLeg import LegacyRWKV
 
 
 def loadPreQuantized(path, tokenizer=None):
@@ -23,11 +22,10 @@ def loadPreQuantized(path, tokenizer=None):
 
     ops = RWKVCudaQuantOps(
         preQuantized=True, embed=len(weights["blocks.0.ln2.weight"]), layers=(n_layers+1), chunksize=32, target=100, maxQuantTarget=100, useLogFix="logfix" in path, useGPU=torch.cuda.is_available(), runtimedtype=torch.float32)
-    if "logfix" in path:
-        model = LegacyRWKV(ops, weights)
-    else:
-        model = AgnostigRWKV(ops, weights)
+    model = AgnosticRWKV(ops, weights)
     emptyState = ops.emptyState
     initTensor = ops.initTensor
+    intTensor = ops.intTensor
+    sample = ops.sample
 
-    return RWKVMaster(model, emptyState, initTensor, ops.sample, tokenizer)
+    return RWKVMaster(model, emptyState, initTensor, intTensor, sample, tokenizer)
