@@ -1,4 +1,4 @@
-import torch
+
 import tqdm
 import rwkvstic.tokenizer as tokenizer
 from typing import List
@@ -7,37 +7,11 @@ from typing import List
 import time
 
 
-def loadContext(model, ctx, newctx, statex, progressCallBack=lambda x: x):
-    tt = time.time()
-    nnewctx = newctx
-    btch = 20
-    o = (None, statex)
 
-    while len(newctx) > 0:
-        m = newctx[:btch]
-        newctx = newctx[btch:]
-        o = model.forward(m, o[1])
-        progressCallBack(m)
-
-    print("loaded context in", time.time()-tt, "seconds")
-    print(o[0][0])
-    return nnewctx, o[1]
-
-
-def rnnloadContext(model, ctx, newctx, statex, progressCallBack=lambda x: x):
-
-    with torch.jit.optimized_execution(True):
-        for i in tqdm.tqdm(range(len(newctx))):
-
-            x = ctx+newctx[:i]
-
-            o = model.forward([x[-1]], statex)
-            statex = o[1]
-            progressCallBack(x)
-    return ctx+newctx, o[1]
 
 
 class RWKVMaster():
+
     def __init__(self, model, emptyState, initTensor=lambda x: x, intTensor=lambda x: x, sampler=None, tokPath=None):
         self.model = model
 
@@ -49,8 +23,8 @@ class RWKVMaster():
         self.initTensor = initTensor
         self.intTensor = intTensor
         self.sampler = sampler
-
-    def forward(self, state=None, temp: float = 1.0, top_p_usual: float = 0.8, number=1, stopStrings: List[str] = ["<|endoftext|>"], stopTokens: List[int] = [0], progressLambda=lambda args: args, end_adj=0.0):
+    logging=False
+    def forward(self, state=None, temp: float = 1.0, top_p_usual: float = 0.8, number=1, stopStrings: List[str] = ["<|endoftext|>"], stopTokens: List[int] = [0], progressLambda=(lambda args: print(args["current"],end="",flush=True)) if logging else (lambda x:x), end_adj=0.0):
         ostate = self.myState if state is None else state
         tolens = []
         for i in range(number):
@@ -63,6 +37,7 @@ class RWKVMaster():
             self.myState = ostate
             sampled = self.sample(
                 logits, temp, top_p_usual) if self.sampler is not None else logits
+            
             try:
                 self.lastToken = [sampled.cpu().numpy()[0]]
             except:
@@ -79,18 +54,52 @@ class RWKVMaster():
 
         return {"logits": logits, "state": ostate, "output": sampled}
 
-    def loadContext(self, newctx: str = "", ctx: str = "\n\n", statex=None, progressCallBack=lambda x: x):
+    def loadContext(self, newctx: str = "", ctx: str = "\n\n", statex=None, progressCallBack=lambda x: x, batch=20):
+
+        def doContext(model, ctx, newctx, statex, progressCallBack=lambda x: x):
+            tt = time.time()
+            nnewctx = newctx
+            ll = len(newctx)
+            btch = batch
+            o = (None, statex)
+
+            while len(newctx) > 0:
+                print(len(newctx)/ll * 100, "%", "remaining")
+                m = newctx[:btch]
+                newctx = newctx[btch:]
+                o = model.forward(m, o[1])
+                progressCallBack(m)
+                
+
+            # print("loaded context in", time.time()-tt, "seconds")
+            # print(o[0][0])
+            return nnewctx, o[1]
+
+
+        def rnndoContext(model, ctx, newctx, statex, progressCallBack=lambda x: x):
+
+            for i in tqdm.tqdm(range(len(newctx))):
+
+                x = ctx+newctx[:i]
+
+                o = model.forward([x[-1]], statex)
+                statex = o[1]
+                progressCallBack(x)
+            return ctx+newctx, o[1]
+
+
         statex = self.myState if statex is None else statex
-        print(newctx)
+        # print(newctx)
         ctx = self.tokenizer.encode(ctx)
         newctx = self.tokenizer.encode(newctx)
-        if self.model.RnnOnly:
-            ctx, state = rnnloadContext(
+        self.lastToken = [ctx[-1]]
+        if self.model.__dict__.get("RnnOnly", False):
+            ctx, state = rnndoContext(
                 self.model, ctx, self.intTensor(newctx), statex, progressCallBack)
         else:
-            ctx, state = loadContext(
+            ctx, state = doContext(
                 self.model, ctx, self.intTensor(newctx), statex, progressCallBack)
-        self.lastToken = ctx[-1]
+        
         self.myState = state
         return ctx, state
 
